@@ -64,6 +64,27 @@ libxml_errors (void *ctx ATTRIBUTE_UNUSED, const char *msg, ...)
 static xmlParserCtxt *ctxt;
 static int libxml2_initialized = 0;
 
+static void
+cleanupLibxml ()
+{
+  destroy_semantic_table ();
+  if (ud->doc != NULL)
+    xmlFreeDoc (ud->doc);
+  if (!libxml2_initialized)
+    return;
+  xmlCleanupParser ();
+  initGenericErrorDefaultFunc (NULL);
+  xmlFreeParserCtxt (ctxt);
+}
+
+static void
+freeEverything ()
+{
+  lou_logEnd ();
+  cleanupLibxml ();
+  lbu_free ();
+}
+
 static int
 processXmlDocument (const char *inputDoc, int length)
 {
@@ -97,102 +118,114 @@ processXmlDocument (const char *inputDoc, int length)
 	  else
 	    ud->doc = xmlParseFile (inputDoc);
 	  if (ud->doc == NULL)
-	    ud->doc = htmlParseFile (inputDoc, NULL);
+	    {
+	      lou_logPrint
+		("Document could not be processed, probably  malformed");
+	      cleanupLibxml ();
+	      return 0;
+	    }
 	}
     }
   else
     ud->doc = xmlParseMemory (inputDoc, length);
   if (ud->doc == NULL)
     {
-      lou_logPrint ("Document could not be processed");
-      kill_safely ();
+      lou_logPrint ("Document could not be processed, probably  malformed");
+      cleanupLibxml ();
+      return 0;
     }
-  if (ud->format_for >= utd && strcmp (ud->doc->encoding, "UTF-8") != 0)
+  if (ud->doc->encoding == NULL)
     {
-    lou_logPrint (
-    "This format requires UTF-8 encoding, not '%s'",
-    ud->doc->encoding);
-    kill_safely ();
+      lou_logPrint ("Encoding, preferably UTF-8,  must be specified");
+      cleanupLibxml ();
+      return 0;
+    }
+  if (ud->format_for >= utd && ignore_case_comp (ud->doc->encoding,
+						 "UTF-8", 5) != 0)
+    {
+      lou_logPrint ("UTDML requires UTF-8 encoding, not '%s'",
+		    ud->doc->encoding);
+      cleanupLibxml ();
+      return 0;
     }
   rootElement = xmlDocGetRootElement (ud->doc);
   if (rootElement == NULL)
     {
       lou_logPrint ("Document is empty");
+      cleanupLibxml ();
       return 0;
     }
   if (ud->mode & convertOnly)
-  convert_utd ();
+    convert_utd ();
   else
-  {
-  haveSemanticFile = compile_semantic_table (rootElement);
-  do_xpath_expr ();
-  examine_document (rootElement);
-  append_new_entries ();
-  if (!haveSemanticFile)
-    return 0;
-  transcribe_document (rootElement);
-  }
-  xmlFreeDoc (ud->doc);
-  xmlCleanupParser ();
-  initGenericErrorDefaultFunc (NULL);
-  xmlFreeParserCtxt (ctxt);
+    {
+      haveSemanticFile = compile_semantic_table (rootElement);
+      do_xpath_expr ();
+      examine_document (rootElement);
+      append_new_entries ();
+      /* This will generate a new semantic-action file if none exists 
+       * and newEntries is  yes in the configuration file.
+       * Otherwise it generates a new_enhtries file.*/
+      if (!haveSemanticFile)
+	{
+	  cleanupLibxml ();
+	  return 0;
+	}
+      if (!transcribe_document (rootElement))
+	{
+	  lou_logPrint ("Document could not be transcribed");
+	  cleanupLibxml ();
+	  return 0;
+	}
+    }
+  cleanupLibxml ();
   return 1;
 }
 
-void
-kill_safely ()
-{
-  lou_logPrint ("liblouisutdml is terminating.");
-  lou_logEnd ();
-  if (ud->doc != NULL)
-    xmlFreeDoc (ud->doc);
-  lbu_free ();
-  if (!libxml2_initialized)
-    exit (1);
-  xmlCleanupParser ();
-  initGenericErrorDefaultFunc (NULL);
-  xmlFreeParserCtxt (ctxt);
-  exit (1);
-}
-
 void *EXPORT_CALL
-lbu_initialize (const char *configFileName,
+lbu_initialize (const char *configFileList,
 		const char *logFileName, const char *settingsString)
 {
-  if (!read_configuration_file (configFileName, logFileName,
+  if (!read_configuration_file (configFileList, logFileName,
 				settingsString, 0))
     return NULL;
   return (void *) ud;
 }
 
 int EXPORT_CALL
-lbu_translateString (const char *configFileName,
-		     const char *inbuf, int inlen, widechar *outbuf,
+lbu_translateString (const char *configFileList,
+		     const char *inbuf, int inlen, widechar * outbuf,
 		     int *outlen,
 		     const char *logFileName, const char *settingsString,
 		     unsigned int mode)
 {
 /* Translate the well-formed xml expression in inbuf into braille 
-* according to the specifications in configFileName. If the expression 
+* according to the specifications in configFileList. If the expression 
 * is not well-formed or there are oteer errors, return 0.*/
   int k;
   char *xmlInbuf;
   if (!read_configuration_file
-      (configFileName, logFileName, settingsString, mode))
+      (configFileList, logFileName, settingsString, mode))
+    return 0;
+  if (inbuf == NULL || outbuf == NULL || outlen == NULL)
     return 0;
   ud->inbuf = inbuf;
   ud->inlen = inlen;
-  ud->outbuf = (widechar *)outbuf;
-  ud->outbuf1_len = *outlen;
+  ud->outbuf = outbuf;
+  ud->outlen = *outlen;
   ud->inFile = ud->outFile = NULL;
   for (k = 0; k < inlen; k++)
     if (inbuf[k] > ' ')
       break;
   if (inbuf[k] != '<')
     {
-      transcribe_text_string ();
+      if (ud->format_for == utd)
+	k = utd_transcribe_text_string ();
+      else
+	k = transcribe_text_string ();
       *outlen = ud->outlen_so_far;
-      return 1;
+      lou_logEnd ();
+      return k;
     }
   if (inbuf[k + 1] == '?')
     xmlInbuf = (char *) inbuf;
@@ -208,28 +241,29 @@ lbu_translateString (const char *configFileName,
       strcat (xmlInbuf, "\n");
       strcat (xmlInbuf, inbuf);
     }
-  ud->inFile = ud->outFile = NULL;
-  if (!processXmlDocument (xmlInbuf, inlen))
-    return 0;
+  k = processXmlDocument (xmlInbuf, inlen);
   *outlen = ud->outlen_so_far;
   if (xmlInbuf != inbuf)
     free (xmlInbuf);
   lou_logEnd ();
-  return 1;
+  return k;
 }
 
 int
   EXPORT_CALL lbu_translateFile
-  (const char *configFileName, const char *inFileName,
+  (const char *configFileList, const char *inFileName,
    const char *outFileName, const char *logFileName,
    const char *settingsString, unsigned int mode)
 {
 /* Translate the well-formed xml expression in inFileName into 
-* braille according to the specifications in configFileName. If the 
+* braille according to the specifications in configFileList. If the 
 * expression is not well-formed or there are other errors, 
 * return 0. */
+  int k;
   if (!read_configuration_file
-      (configFileName, logFileName, settingsString, mode))
+      (configFileList, logFileName, settingsString, mode))
+    return 0;
+  if (inFileName == NULL || outFileName == NULL)
     return 0;
   if (strcmp (outFileName, "stdout"))
     {
@@ -241,26 +275,28 @@ int
     }
   else
     ud->outFile = stdout;
-  if (!processXmlDocument (inFileName, 0))
-    return 0;
+  k = processXmlDocument (inFileName, 0);
   if (ud->outFile != stdout)
     fclose (ud->outFile);
   lou_logEnd ();
-  return 1;
+  return k;
 }
 
 int
   EXPORT_CALL lbu_translateTextFile
-  (const char *configFileName, const char *inFileName,
+  (const char *configFileList, const char *inFileName,
    const char *outFileName, const char *logFileName,
    const char *settingsString, unsigned int mode)
 {
 /* Translate the text file in inFileName into braille according to
-* the specifications in configFileName. If there are errors, print 
+* the specifications in configFileList. If there are errors, print 
 * an error message and return 0.*/
+  int k;
   widechar outbuf[2 * BUFSIZE];
   if (!read_configuration_file
-      (configFileName, logFileName, settingsString, mode))
+      (configFileList, logFileName, settingsString, mode))
+    return 0;
+  if (inFileName == NULL || outFileName == NULL)
     return 0;
   ud->outbuf = outbuf;
   ud->outlen = (sizeof (outbuf) / CHARSIZE) - 4;
@@ -268,7 +304,7 @@ int
     {
       if (!(ud->inFile = fopen (inFileName, "rb")))
 	{
-	  lou_logPrint ("Can't open input file %s.\n", inFileName);
+	  lou_logPrint ("Can't open input file %s.", inFileName);
 	  return 0;
 	}
     }
@@ -278,40 +314,57 @@ int
     {
       if (!(ud->outFile = fopen (outFileName, "wb")))
 	{
-	  lou_logPrint ("Can't open output file %s.\n", outFileName);
+	  lou_logPrint ("Can't open output file %s.", outFileName);
 	  return 0;
 	}
     }
   else
     ud->outFile = stdout;
-  transcribe_text_file ();
+  if (ud->format_for == utd)
+    k = utd_transcribe_text_file ();
+  else
+    k = transcribe_text_file ();
+  if (!k)
+    {
+      freeEverything ();
+      return 0;
+    }
   if (ud->inFile != stdin)
     fclose (ud->inFile);
   if (ud->outFile != stdout)
     fclose (ud->outFile);
   lou_logEnd ();
-  return 1;
+  return k;
 }
 
 int EXPORT_CALL
-lbu_backTranslateString (const char *configFileName,
+lbu_backTranslateString (const char *configFileList,
 			 const char *inbuf, int inlen, widechar
-			 *outbuf,
+			 * outbuf,
 			 int *outlen,
 			 const char *logFileName, const char
 			 *settingsString, unsigned int mode)
 {
+  int k;
   if (!read_configuration_file
-      (configFileName, logFileName, settingsString, mode))
+      (configFileList, logFileName, settingsString, mode))
+    return 0;
+  if (inbuf == NULL || outbuf == NULL || outlen == NULL)
     return 0;
   ud->inbuf = inbuf;
   ud->inlen = inlen;
-  ud->outbuf = (widechar *)outbuf;
-  ud->outbuf1_len = *outlen;
-  ud->outbuf = (widechar *) outbuf;
-  ud->outlen = ud->outlen / CHARSIZE;
+  ud->outbuf = outbuf;
+  ud->outlen = *outlen;
   ud->inFile = ud->outFile = NULL;
-  back_translate_braille_string ();
+  if (ud->format_for == utd)
+    k = utd_back_translate_braille_string ();
+  else
+    k = back_translate_braille_string ();
+  if (!k)
+    {
+      freeEverything ();
+      return 0;
+    }
   *outlen = ud->outlen_so_far;
   lou_logEnd ();
   return 1;
@@ -319,17 +372,20 @@ lbu_backTranslateString (const char *configFileName,
 
 int
   EXPORT_CALL lbu_backTranslateFile
-  (const char *configFileName, const char *inFileName,
+  (const char *configFileList, const char *inFileName,
    const char *outFileName, const char *logFileName,
    const char *settingsString, unsigned int mode)
 {
 /* Back translate the braille file in inFileName into either an 
 * xml file or a text file according to
-* the specifications in configFileName. If there are errors, print an 
+* the specifications in configFileList. If there are errors, print an 
 * error message and return 0.*/
+  int k;
   widechar outbuf[2 * BUFSIZE];
   if (!read_configuration_file
-      (configFileName, logFileName, settingsString, mode))
+      (configFileList, logFileName, settingsString, mode))
+    return 0;
+  if (inFileName == NULL || outFileName == NULL)
     return 0;
   ud->outbuf = outbuf;
   ud->outlen = (sizeof (outbuf) / CHARSIZE) - 4;
@@ -337,7 +393,7 @@ int
     {
       if (!(ud->inFile = fopen (inFileName, "rb")))
 	{
-	  lou_logPrint ("Can't open input file %s.\n", inFileName);
+	  lou_logPrint ("Can't open input file %s.", inFileName);
 	  return 0;
 	}
     }
@@ -347,13 +403,21 @@ int
     {
       if (!(ud->outFile = fopen (outFileName, "wb")))
 	{
-	  lou_logPrint ("Can't open output file %s.\n", outFileName);
+	  lou_logPrint ("Can't open output file %s.", outFileName);
 	  return 0;
 	}
     }
   else
     ud->outFile = stdout;
-  back_translate_file ();
+  if (ud->format_for == utd)
+    k = utd_back_translate_file ();
+  else
+    k = back_translate_file ();
+  if (!k)
+    {
+      freeEverything ();
+      return 0;
+    }
   if (ud->inFile != stdin)
     fclose (ud->inFile);
   if (ud->outFile != stdout)
@@ -368,6 +432,9 @@ static char *writeablePathPtr = NULL;
 char *EXPORT_CALL
 lbu_setWriteablePath (const char *path)
 {
+  writeablePathPtr = NULL;
+  if (path == NULL)
+    return NULL;
   strcpy (writeablePath, path);
   writeablePathPtr = writeablePath;
   return writeablePathPtr;
@@ -388,6 +455,8 @@ lbu_charToDots (const char *tableList, const unsigned char *inbuf,
   int wcLength;
   int utf8Length;
   int result = 0;
+  if (tableList == NULL || inbuf == NULL || outbuf == NULL)
+    return 0;
   lou_logFile (logFile);
   interBuf = malloc (length * CHARSIZE);
   utf8Length = length;
@@ -414,6 +483,8 @@ lbu_dotsToChar (const char *tableList, const unsigned char *inbuf,
   int wcLength;
   int utf8Length;
   int result = 0;
+  if (tableList == NULL || inbuf == NULL || outbuf == NULL)
+    return 0;
   lou_logFile (logFile);
   interBuf = malloc (length * CHARSIZE);
   utf8Length = length;
@@ -442,7 +513,7 @@ lbu_checkTable (const char *tableList, const char *logFile, unsigned int mode)
 }
 
 void EXPORT_CALL
-lbu_free (void)
+lbu_free ()
 {
 /* Free all memory used by liblouisutdml. You MUST call this function at 
 * the END of your application.*/
