@@ -32,8 +32,8 @@
 #ifndef liblouisutdml_h
 #define liblouisutdml_h
 #include <libxml/parser.h>
-#include <louis.h>
 #include "liblouisutdml.h"
+#include <louis.h>
 #include "sem_enum.h"
 
 typedef enum
@@ -46,6 +46,7 @@ typedef enum
 
 typedef enum
 {
+  inherit = -100,
   leftJustified = 0,
   rightJustified = 1,
   centered = 2,
@@ -71,7 +72,7 @@ typedef struct
   int lines_before;
   int lines_after;
   int left_margin;
-  int centered_margin;
+  int right_margin;
   int keep_with_next;
   int dont_split;
   int orphan_control;
@@ -84,6 +85,8 @@ typedef struct
   int newpage_after;
   int righthand_page;
   int newline_after;
+  int runningHead;
+  char name[1];
 } StyleType;
 
 #ifdef _WIN32
@@ -96,9 +99,10 @@ typedef struct
 #endif
 
 #define BUFSIZE 8192
+#define BIG_BUFSIZE 4 * BUFSIZE - 8
 #define MAX_LENGTH BUFSIZE - 4
 #define MAX_TRANS_LENGTH 2 * BUFSIZE - 4
-#define MAXNAMELEN 256
+#define MAXNAMELEN 1024
 #define MAXNUMLEN 32
 #define STACKSIZE 100
 #define MAXLINES 512
@@ -127,7 +131,8 @@ typedef enum
   volumes,
   brf,
   volumesPef,
-  volumesBrf
+  volumesBrf,
+  dsbible
 } FormatFor;
 
 typedef enum
@@ -146,6 +151,10 @@ typedef struct
   xmlNode *node;
   StyleStatus status;
   BrlPageNumFormat curBrlNumFormat;
+  StyleFormat curStyleFormat;
+  int curLeftMargin;
+  int curRightMargin;
+  int curFirstLineIndent;
 } StyleRecord;
 
 typedef struct
@@ -158,6 +167,7 @@ typedef struct
   int old_text_length;
   int translated_length;
   int string_buf_len;
+  int needs_editing;
   int has_math;
   int has_comp_code;
   int has_chem;
@@ -224,7 +234,6 @@ typedef struct
   int outbuf3_enabled;
   int fill_pages;
   int after_contents;
-  int fill_page_skipped;
   int blank_lines;
   int print_page_numbers_in_contents;
   int braille_page_numbers_in_contents;
@@ -257,6 +266,8 @@ typedef struct
   char lit_hyphen[5];
   char comp_hyphen[5];
   char letsign[5];
+  char *config_path;
+  char *lbu_files_path;
   widechar running_head[MAXNAMELEN / 2];
   widechar footer[MAXNAMELEN / 2];
   int running_head_length;
@@ -268,10 +279,8 @@ typedef struct
   const char *mathexpr_table_name;
   const char *edit_table_name;
   const char *semantic_files;
-  const char *volume_sem;
-  const char *pef_sem;
-  const char *brf_sem;
-  const char *transinxml_sem;
+  const char *converter_sem;
+  const char *pass2_conv_sem;
   widechar print_page_number[MAXNUMLEN];
   widechar braille_page_string[MAXNUMLEN];
   char lineEnd[8];
@@ -283,8 +292,16 @@ typedef struct
   StyleRecord style_stack[STACKSIZE];
   int style_top;
   BrlPageNumFormat brl_page_num_format;
+  StyleFormat style_format;
+  int style_left_margin;
+  int style_right_margin;
+  int style_first_line_indent;
   char xml_header[2 * MAXNAMELEN];
   widechar text_buffer[2 * BUFSIZE];
+  int in_sync;
+  widechar sync_text_buffer[2 * BUFSIZE];
+  int sync_text_length;
+  int positions_array[2 * BUFSIZE];
   widechar translated_buffer[2 * BUFSIZE];
   unsigned char typeform[2 * BUFSIZE];
   char string_buffer[2 * BUFSIZE];
@@ -327,6 +344,7 @@ int finish_heading (sem_act action);
 int make_contents ();
 void do_reverse (xmlNode * node);
 int do_boxline (xmlNode * node);
+void do_pagebreak (xmlNode *node);
 void do_linespacing (xmlNode * node);
 int do_newpage ();
 int do_blankline ();
@@ -367,11 +385,27 @@ int wc_string_to_utf8 (const widechar * instr, int *inSize, unsigned
 		       char *outstr, int *outSize);
 void output_xml (xmlDoc *doc);
 int convert_utd ();
+int utd2bible (xmlNode * node);
 int utd2brf (xmlNode * node);
 int utd2pef (xmlNode * node);
 int utd2transinxml (xmlNode * node);
 int utd2volumes (xmlNode * node);
 int hasAttrValue  (xmlNode *node, char *attrName, char *value);
-void kill_safely ();
-
+xmlChar * new_macro (xmlChar * name, xmlChar *body);
+unsigned char * lookup_macro (xmlChar * name);
+char *is_macro (xmlNode *node);
+int start_macro (xmlNode *node);
+int end_macro ();
+int ignore_case_comp (const char *str1, const char *str2, int length);
+void insert_text_string (xmlNode *node, xmlChar *str);
+void do_attrtotext (xmlNode *node);
+int utd_transcribe_text_file ();
+int utd_transcribe_text_string ();
+int utd_back_translate_braille_string ();
+int utd_back_translate_file ();
+int pass2_conv ();
+void memoryError ();
+unsigned char *get_sem_name (xmlNode *node);
+void set_runninghead_string (widechar *chars, int length);
+void set_footer_string (widechar *chars, int length);
 #endif /*louisutdml_h */
