@@ -55,24 +55,42 @@ static int saved_braillePageNumber;
 static widechar saved_printPageNumber[MAXNUMLEN];
 static widechar saved_printPageNumberFirst[MAXNUMLEN];
 static widechar saved_printPageNumberLast[MAXNUMLEN];
+static widechar saved_pageSeparatorNumberFirst[MAXNUMLEN];
+static widechar saved_pageSeparatorNumberLast[MAXNUMLEN];
+static int saved_runningHeadLength;
+static int saved_footerLength;
+static widechar saved_runningHead[MAXNAMELEN / 2];
+static widechar saved_footer[MAXNAMELEN / 2];
 static BrlPageNumFormat saved_braillePageNumberFormat;
 static StyleRecord *styleSpec;
+static SaveHeading *saved_firstHeading;
+static SaveHeading *saved_lastHeading;
+
+static void free_headings (SaveHeading *start);
 
 int
 initialize_contents (void)
 {
 
   saved_braillePageNumberFormat = ud->brl_page_num_format;
-  widestrcpy (saved_printPageNumber, ud->print_page_number, -1);
-  widestrcpy (saved_printPageNumberFirst, ud->print_page_number_first, -1);
-  widestrcpy (saved_printPageNumberLast, ud->print_page_number_last, -1);
+  widestrcpy (saved_printPageNumber, ud->print_page_number);
+  widestrcpy (saved_printPageNumberFirst, ud->print_page_number_first);
+  widestrcpy (saved_printPageNumberLast, ud->print_page_number_last);
+  widestrcpy (saved_pageSeparatorNumberFirst, ud->page_separator_number_first);
+  widestrcpy (saved_pageSeparatorNumberLast, ud->page_separator_number_last);
+  saved_runningHeadLength = ud->running_head_length;
+  saved_footerLength = ud->footer_length;
+  widecharcpy (saved_runningHead, ud->running_head, ud->running_head_length);
+  widecharcpy (saved_footer, ud->footer, ud->footer_length);
   ud->after_contents = 1;
   saved_udContents = ud->contents;
   saved_linesOnPage = ud->lines_on_page;
   saved_braillePageNumber = ud->braille_page_number;
   ud->contents = 1;
   firstHeading = NULL;
-  lastHeading = &heading;
+  lastHeading = NULL;
+  saved_lastHeading = NULL;
+  saved_firstHeading = NULL;
   saved_outFile = ud->outFile;
   strcpy (tempFileName, lbu_getWriteablePath ());
   strcat (tempFileName, "lbx_body.temp");
@@ -162,7 +180,8 @@ finish_heading (sem_act action)
   headingSize += heading.headingLength * CHARSIZE;
   headingPtr = malloc (headingSize);
   memcpy (headingPtr, &heading, headingSize);
-  lastHeading->next = headingPtr;
+  if (lastHeading != NULL)
+    lastHeading->next = headingPtr;
   lastHeading = headingPtr;
   if (firstHeading == NULL)
     firstHeading = headingPtr;
@@ -189,10 +208,15 @@ make_contents (void)
       styleSpec = &ud->style_stack[ud->style_top];
       styleSpec->curBrlNumFormat = saved_braillePageNumberFormat;
       ud->brl_page_num_format = saved_braillePageNumberFormat;
-      widestrcpy (ud->print_page_number, saved_printPageNumber, -1);
-      widestrcpy (ud->print_page_number_first, saved_printPageNumberFirst,
-		  -1);
-      widestrcpy (ud->print_page_number_last, saved_printPageNumberLast, -1);
+      widestrcpy (ud->print_page_number, saved_printPageNumber);
+      widestrcpy (ud->print_page_number_first, saved_printPageNumberFirst);
+      widestrcpy (ud->print_page_number_last, saved_printPageNumberLast);
+      widestrcpy (ud->page_separator_number_first, saved_pageSeparatorNumberFirst);
+      widestrcpy (ud->page_separator_number_last, saved_pageSeparatorNumberLast);
+      ud->running_head_length = saved_runningHeadLength;
+      ud->footer_length = saved_footerLength;
+      widecharcpy (ud->running_head, saved_runningHead, ud->running_head_length);
+      widecharcpy (ud->footer, saved_footer, ud->footer_length);
       do_newpage ();
       ud->contents = 2;
       currentHeading = firstHeading;
@@ -240,20 +264,15 @@ make_contents (void)
 	  memcpy (ud->translated_buffer, currentHeading->headingChars,
 		  currentHeading->headingLength * CHARSIZE);
 	  ud->translated_length = currentHeading->headingLength;
+	  ud->in_sync = 0;
 	  end_style (style);
 	  currentHeading = currentHeading->next;
 	}
       do_newpage ();
       ud->prelim_pages = ud->braille_page_number;
       ud->braille_page_number = saved_braillePageNumber;
-      /*Free headings */
-      currentHeading = firstHeading;
-      while (currentHeading->next != NULL)
-	{
-	  lastHeading = currentHeading;
-	  currentHeading = currentHeading->next;
-	  free (lastHeading);
-	}
+      free_headings (firstHeading);
+      firstHeading = NULL;
       ud->contents = saved_udContents;
       ud->braille_page_number = old_braillePageNumber;
     }
@@ -273,3 +292,32 @@ make_contents (void)
   fclose (tempFile);
   return 1;
 }
+
+void
+contents_save_state ()
+{
+  saved_firstHeading = firstHeading;
+  saved_lastHeading = lastHeading;
+}
+
+void
+contents_restore_state ()
+{
+  firstHeading = saved_firstHeading;
+  lastHeading = saved_lastHeading;
+  if (lastHeading != NULL) {
+    free_headings (lastHeading->next);
+    lastHeading->next = NULL; }
+}
+
+static void
+free_headings (SaveHeading *start)
+{
+  SaveHeading *current;
+  SaveHeading *next;
+  if (start == NULL) return;
+  current = start;
+  while ((next = current->next) != NULL) {
+    free(current);
+    current = next; }
+}
