@@ -29,6 +29,7 @@
    Maintained by John J. Boyer john.boyer@jjb-software.com
    */
 
+#include <stddef.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -38,6 +39,16 @@
 #include <louis.h>
 #include "louisutdml.h"
 #include "sem_names.h"
+#include <ctype.h>
+
+void
+memoryError ()
+{
+  fprintf (stderr,
+  "liblouisutdml: insufficient memory\n");
+  //"liblouisutdml: Insufficient memory: %s", strerror (errno), "\n");
+  exit (3);
+}
 
 typedef struct
 {
@@ -56,7 +67,7 @@ typedef struct
 }
 FileInfo;
 
-static char lastConfigFileList[MAXNAMELEN];
+static char pathEnd[2];
 static double paperWidth;
 static double paperHeight;
 static double leftMargin;
@@ -64,6 +75,7 @@ static double rightMargin;
 static double topMargin;
 static double bottomMargin;
 static int errorCount = 0;
+static int fatalErrorCount = 0;
 
 static void
 configureError (FileInfo * nested, char *format, ...)
@@ -90,8 +102,8 @@ alloc_string (const char *inString)
   int length;
   char *newString;
   if (inString == NULL)
-  return NULL;
-  length  = strlen (inString);
+    return NULL;
+  length = strlen (inString);
   if ((length + ud->string_buf_len) >= sizeof (ud->string_buffer))
     return NULL;
   newString = &ud->string_buffer[ud->string_buf_len];
@@ -105,7 +117,7 @@ alloc_string_if_not (const char *inString)
 {
   int alreadyStored;
   if (inString == NULL)
-  return NULL;
+    return NULL;
   alreadyStored = inString - ud->string_buffer;
   if (alreadyStored >= 0 && alreadyStored < ud->string_buf_len)
     return (char *) inString;
@@ -131,9 +143,6 @@ find_file (const char *fileList, char *filePath)
   int k;
   int currentListPos = 0;
   int pathLength;
-  char pathEnd[2];
-  pathEnd[0] = ud->file_separator;
-  pathEnd[1] = 0;
   filePath[0] = 0;
   for (commaPos = 0; fileList[commaPos] && fileList[commaPos] != ',';
        commaPos++);
@@ -199,10 +208,35 @@ static char *
 findTable (FileInfo * nested)
 {
   char trialPath[MAXNAMELEN];
-  if (!find_file (nested->value, trialPath))
+  char filePath[MAXNAMELEN];
+  struct stat statInfo;
+  filePath[0] = 0;
+  if (ud->config_path != NULL)
     {
-      configureError (nested, "Table '%s' cannot be found.", 
-      nested->value);
+      strcpy (trialPath, ud->config_path);
+      if (trialPath[strlen (trialPath) -1] != ud->file_separator)
+        strcat (trialPath, pathEnd);
+      strcat (trialPath, nested->value);
+      if (stat (trialPath, &statInfo) != -1)
+	strcpy (filePath, trialPath);
+    }
+  else if (filePath[0] == 0)
+    {
+      strcpy (trialPath, ud->lbu_files_path);
+      strcat (trialPath, nested->value);
+      if (stat (trialPath, &statInfo) != -1)
+	strcpy (filePath, trialPath);
+    }
+  strcpy (trialPath, filePath);
+  if (trialPath[0] == 0)
+    {
+      if (lou_getTable (nested->value) != NULL)
+	strcpy (trialPath, getLastTableList ());
+    }
+  if (trialPath[0] == 0)
+    {
+      configureError (nested, "Table '%s' cannot be found.", nested->value);
+      fatalErrorCount++;
       return NULL;
     }
   return alloc_string_if_not (trialPath);
@@ -237,8 +271,7 @@ controlCharValue (FileInfo * nested)
 	      decoded[decodedLength++] = 13;
 	      break;
 	    default:
-	      configureError (nested, "invalid value '%s'", 
-	      nested->value);
+	      configureError (nested, "invalid value '%s'", nested->value);
 	      return 0;
 	    }
 	  k++;
@@ -269,8 +302,7 @@ config_compileSettings (const char *fileName)
     return compileConfig (&nested);
   if (!find_file (fileName, completePath))
     {
-      configureError (NULL, "Can't find configuration file '%s'", 
-      fileName);
+      configureError (NULL, "Can't find configuration file '%s'", fileName);
       return 0;
     }
   if ((nested.in = fopen ((char *) completePath, "rb")))
@@ -280,8 +312,7 @@ config_compileSettings (const char *fileName)
     }
   else
     {
-      configureError (NULL, "Can't open configuration file '%s'", 
-      fileName);
+      configureError (NULL, "Can't open configuration file '%s'", fileName);
       return 0;
     }
   return 1;
@@ -332,9 +363,9 @@ parseLine (FileInfo * nested)
 {
   char *curchar = NULL;
   int ch = 0;
-  nested->action = NULL;
-  nested->value = NULL;
-  nested->value2 = NULL;
+  nested->action = "";
+  nested->value = "";
+  nested->value2 = "";
   while (getLine (nested))
     {
       nested->lineNumber++;
@@ -343,13 +374,13 @@ parseLine (FileInfo * nested)
       if (ch == 0 || ch == '#' || ch == '<')
 	continue;
       nested->action = curchar - 1;
-      while ((ch = *curchar++) > 32);
+      while ((ch = *curchar++) > 32 && ch != '=');
       nested->actionLength = curchar - nested->action - 1;
       nested->action[nested->actionLength] = 0;
-      while ((ch = *curchar++) <= 32 && ch != 0);
+      while (((ch = *curchar++) <= 32 || ch == '=') && ch != 0);
       if (ch == 0)
 	{
-	  nested->value = NULL;
+	  nested->value = "";
 	  return 1;
 	}
       else
@@ -388,7 +419,7 @@ parseLine (FileInfo * nested)
 	  nested->value2[nested->value2Length] = 0;
 	}
       else
-	nested->value2 = NULL;
+	nested->value2 = "";
       return 1;
     }
   return 0;
@@ -399,8 +430,8 @@ static int mainActionNumber = NOTFOUND;
 static int subActionNumber;
 static int entities = 0;
 
-static int
-ignoreCaseComp (const char *str1, const char *str2, int length)
+int
+ignore_case_comp (const char *str1, const char *str2, int length)
 {
 /* Replaces strncasecmp, which some compilers don't support */
   int k;
@@ -419,7 +450,7 @@ find_action (const char **actions, const char *action)
   int k;
   for (k = 0; actions[k]; k += 2)
     if (actionLength == strlen (actions[k])
-	&& ignoreCaseComp (actions[k], action, actionLength) == 0)
+	&& ignore_case_comp (actions[k], action, actionLength) == 0)
       break;
   if (actions[k] == NULL)
     return -1;
@@ -441,7 +472,7 @@ checkValues (FileInfo * nested, const char **values)
   int k;
   for (k = 0; values[k]; k += 2)
     if (nested->valueLength == strlen (values[k]) &&
-	ignoreCaseComp (values[k], nested->value, nested->valueLength) == 0)
+	ignore_case_comp (values[k], nested->value, nested->valueLength) == 0)
       break;
   if (values[k] == NULL)
     {
@@ -479,7 +510,7 @@ hexValue (FileInfo * nested, const char *digits)
 }
 
 static unsigned int
-convertValue (FileInfo *nested, const char *number)
+convertValue (FileInfo * nested, const char *number)
 {
   if (number[0] == '0' && number[1] == 'x')
     return hexValue (nested, &number[2]);
@@ -501,13 +532,12 @@ orValues (FileInfo * nested, const char **values)
   int wordLength;
   while (word < nested->valueLength)
     {
-      for (wordLength = 0; (word + wordLength) < nested->valueLength && 
-      nested->value[word + wordLength] 
-      > ' '
-	   && nested->value[word + wordLength] != ','; wordLength++);
+      for (wordLength = 0; (word + wordLength) < nested->valueLength &&
+	   nested->value[word + wordLength]
+	   > ' ' && nested->value[word + wordLength] != ','; wordLength++);
       for (k = 0; values[k]; k += 2)
 	if (wordLength == strlen (values[k]) &&
-	    ignoreCaseComp (values[k], &nested->value[word], wordLength) == 0)
+	    ignore_case_comp (values[k], &nested->value[word], wordLength) == 0)
 	  {
 	    result |= convertValue (nested, values[k + 1]);
 	    break;
@@ -530,7 +560,7 @@ checkSubActions (FileInfo * nested, const char **mainActions, const char
   int subAction;
   mainActionNumber = NOTFOUND;
   subAction = checkActions (nested, subActions);
-  if (subAction != NOTFOUND && nested->value == NULL)
+  if (subAction != NOTFOUND && nested->value == "")
     {
       configureError (nested, "column 2 is required");
       return NOTFOUND;
@@ -539,8 +569,8 @@ checkSubActions (FileInfo * nested, const char **mainActions, const char
     {
       mainActionNumber = checkActions (nested, mainActions);
       if (mainActionNumber == NOTFOUND)
-	configureError (nested, 
-	"word '%s' in first column not recognized",
+	configureError (nested,
+			"word '%s' in column 1 not recognized",
 			nested->action);
       return NOTFOUND;
     }
@@ -631,8 +661,9 @@ compileConfig (FileInfo * nested)
     "35",
     "leftMargin",
     "36",
-// available    "37",
-    "volumeSem",
+    "pass2convsem",
+    "37",
+    "converterSem",
     "38",
     "braillePageNumber",
     "39",
@@ -670,12 +701,11 @@ compileConfig (FileInfo * nested)
     "55",
     "mode",
     "56",
-    "pefSem",
-    "57",
-    "transinxmlSem",
-    "58",
-    "brfSem",
-    "59",
+// available    "57",
+// available     "58",
+// available    "59",
+    "macro",
+    "60",
     "style",
     "90",
     NULL
@@ -692,6 +722,9 @@ compileConfig (FileInfo * nested)
   static const char *backFormats[] = {
     "plain", "0", "html", "1", NULL
   };
+  static const char *hyphenationModes[] = {
+      "no", "0", "yes", "1", "pre", "2", NULL
+  };
 
   static const char *formatFor[] = {
     "textDevice", "0",
@@ -701,8 +734,9 @@ compileConfig (FileInfo * nested)
     "transInXml", "4",
     "volumes", "5",
     "brf", "6",
-    "volumesPef", "7"
+    "volumesPef", "7", 
     "volumesBrf", "8",
+    "dsbible", "9",
     NULL
   };
 
@@ -732,6 +766,7 @@ compileConfig (FileInfo * nested)
     "notSync", "1<<27",
     "utdInput", "1<<26",
     "convertOnly", "1<<25",
+    "louisDots", "1<<24",
     NULL
   };
 
@@ -742,7 +777,7 @@ compileConfig (FileInfo * nested)
       if (mainActionNumber == NOTFOUND)
 	{
 	  configureError (nested,
-			  "word '%s' in first column not recognized",
+			  "word '%s' in column 1 not recognized",
 			  nested->action);
 	  continue;
 	}
@@ -805,7 +840,7 @@ compileConfig (FileInfo * nested)
 	    }
 	  break;
 	case 13:
-	  if ((k = checkValues (nested, yesNo)) != NOTFOUND)
+	  if ((k = checkValues (nested, hyphenationModes)) != NOTFOUND)
 	    ud->hyphenate = k;
 	  break;
 	case 14:
@@ -821,21 +856,51 @@ compileConfig (FileInfo * nested)
 	  break;
 	case 18:
 	  ud->contracted_table_name = findTable (nested);
+	  if (ud->contracted_table_name == NULL)
+	  {
+	  configureError (nested, "invalid literaryTextTable");
+	  return 0;
+	  }
 	  break;
 	case 19:
 	  ud->edit_table_name = findTable (nested);
+	  if (ud->edit_table_name == NULL)
+	  {
+	  configureError (nested, "invalid editTable");
+	  return 0;
+	  }
 	  break;
 	case 20:
 	  ud->uncontracted_table_name = findTable (nested);
+	  if (ud->uncontracted_table_name == NULL)
+	  {
+	  configureError (nested, "invalid uncontractedTableName");
+	  return 0;
+	  }
 	  break;
 	case 21:
 	  ud->compbrl_table_name = findTable (nested);
+	  if (ud->compbrl_table_name == NULL)
+	  {
+	  configureError (nested, "invalid compbrlTableName");
+	  return 0;
+	  }
 	  break;
 	case 22:
 	  ud->mathtext_table_name = findTable (nested);
+	  if (ud->mathtext_table_name == NULL)
+	  {
+	  configureError (nested, "invalid mathtextTableName");
+	  return 0;
+	  }
 	  break;
 	case 23:
 	  ud->mathexpr_table_name = findTable (nested);
+	  if (ud->mathexpr_table_name == NULL)
+	  {
+	  configureError (nested, "invalid mathexprTableName");
+	  return 0;
+	  }
 	  break;
 	case 24:
 	  topMargin = atof (nested->value);
@@ -851,7 +916,7 @@ compileConfig (FileInfo * nested)
 	  strcpy (ud->xml_header, nested->value);
 	  break;
 	case 26:
-	  if (nested->value == NULL)
+	  if (nested->value == "")
 	    break;
 	  if (!entities)
 	    strcat (ud->xml_header, "<!DOCTYPE entities [\n");
@@ -878,7 +943,7 @@ compileConfig (FileInfo * nested)
 	    static const char *actions[] = {
 	      NULL
 	    };
-	    if (nested->value == NULL)
+	    if (nested->value == "")
 	      configureError (nested, "a file name in column 2 is required");
 	    else
 	      config_compileSettings (nested->value);
@@ -902,7 +967,7 @@ compileConfig (FileInfo * nested)
 	    ud->contents = k;
 	  break;
 	case 34:
-	  if (nested->value == NULL)
+	  if (nested->value == "")
 	    ud->line_fill = ' ';
 	  else
 	    ud->line_fill = nested->value[0];
@@ -914,9 +979,11 @@ compileConfig (FileInfo * nested)
 	case 36:
 	  leftMargin = atof (nested->value);
 	  break;
-// available	case 37:
+    case 37:
+      ud->pass2_conv_sem = alloc_string (nested->value);
+      break;
 	case 38:
-	  ud->volume_sem = alloc_string (nested->value);
+	  ud->converter_sem = alloc_string (nested->value);
 	  break;
 	case 39:
 	  ud->braille_page_number = atoi (nested->value);
@@ -987,14 +1054,11 @@ compileConfig (FileInfo * nested)
 	  if ((k = orValues (nested, configModes)) != NOTFOUND)
 	    ud->config_mode = k;
 	  break;
-	case 57:
-	  ud->pef_sem = alloc_string (nested->value);
-	  break;
-	case 58:
-	  ud->transinxml_sem = alloc_string (nested->value);
-	  break;
-	case 59:
-	  ud->brf_sem = alloc_string (nested->value);
+// available	case 57:
+// available	case 58:
+// available	case 59:
+	case 60:
+	  new_macro (nested->value, nested->value2);
 	  break;
 	case 90:
 	  {
@@ -1007,7 +1071,7 @@ compileConfig (FileInfo * nested)
 	      "2",
 	      "firstLineIndent",
 	      "3",
-	      "translattionTable",
+	      "translationTable",
 	      "6",
 	      "skipNumberLines",
 	      "7",
@@ -1021,7 +1085,7 @@ compileConfig (FileInfo * nested)
 	      "11",
 	      "braillePageNumberFormat",
 	      "12",
-	      "centeredMargin",
+	      "rightMargin",
 	      "13",
 	      "keepWithNext",
 	      "14",
@@ -1031,9 +1095,13 @@ compileConfig (FileInfo * nested)
 	      "16",
 	      "newlineAfter",
 	      "17",
+	      "runningHead",
+	      "18",
 	      NULL
 	    };
 	    static const char *formats[] = {
+	      "inherit",
+	      "-100",
 	      "leftJustified",
 	      "0",
 	      "rightJustified",
@@ -1047,27 +1115,27 @@ compileConfig (FileInfo * nested)
 	      "listColumns",
 	      "5",
 	      "listLines",
-	      "6", 
-	      "computerCoded", 
-	      "7", 
-	      "contents", 
-	      "8", 
+	      "6",
+	      "computerCoded",
+	      "7",
+	      "contents",
+	      "8",
 	      NULL
 	    };
 	    static const char *pageNumFormats[] = {
-	      "normal", 
-	      "0", 
-	      "blank", 
-	      "1", 
-	      "p", 
-	      "2", 
-	      "roman", 
-	      "3", 
+	      "normal",
+	      "0",
+	      "blank",
+	      "1",
+	      "p",
+	      "2",
+	      "roman",
+	      "3",
 	      NULL
 	    };
 	    StyleType *style;
 	    sem_act styleAction;
-	    if (nested->value == NULL)
+	    if (nested->value == "")
 	      {
 		configureError (nested,
 				"no style name given in second column");
@@ -1127,7 +1195,7 @@ compileConfig (FileInfo * nested)
 		      style->brlNumFormat = k;
 		    break;
 		  case 13:
-		    style->centered_margin = atoi (nested->value);
+		    style->right_margin = atoi (nested->value);
 		    break;
 		  case 14:
 		    if ((k = checkValues (nested, yesNo)) != NOTFOUND)
@@ -1143,7 +1211,11 @@ compileConfig (FileInfo * nested)
 		  case 17:
 		    if ((k = checkValues (nested, yesNo)) != NOTFOUND)
 		      style->newline_after = k;
-		  break;
+		    break;
+		  case 18:
+		    if ((k = checkValues (nested, yesNo)) != NOTFOUND)
+		      style->runningHead = k;
+		    break;
 		  default:
 		    configureError (nested, "Program error in readconfig.c");
 		    continue;
@@ -1174,17 +1246,14 @@ initConfigFiles (const char *firstConfigFile, char *fileName, const
     k++;
   configPath[k] = 0;
   set_paths (configPath);
-  if (logFileName)
+  if (logFileName && logFileName[0] != 0)
     {
       strcpy ((char *) ud->typeform, lbu_getWriteablePath ());
       strcat ((char *) ud->typeform, logFileName);
       lou_logFile ((char *) ud->typeform);
     }
   if (!config_compileSettings ("liblouisutdml.ini"))
-    {
-      if (!config_compileSettings ("canonical.cfg"))
-	return 0;
-    }
+    return 0;
   return 1;
 }
 
@@ -1200,50 +1269,25 @@ read_configuration_file (const char *configFileList, const char
   int listLength;
   int currentListPos = 0;
   errorCount = 0;
+  fatalErrorCount = 0;
   /*Process logFileName later, after writeablePath is set */
-  if ((configFileList != NULL && strcmp (configFileList,
-					 lastConfigFileList) == 0)
-      && !(mode & doInit))
-    {
-      ud->has_comp_code = 0;
-      ud->has_math = 0;
-      ud->has_chem = 0;
-      ud->has_graphics = 0;
-      ud->has_music = 0;
-      ud->has_cdata = 0;
-      ud->has_contentsheader = 0;
-      ud->prelim_pages = 0;
-      ud->braille_page_string[0] = 0;
-      ud->print_page_number[0] = '_';
-      ud->inFile = NULL;
-      ud->outFile = NULL;
-      ud->main_braille_table = ud->contracted_table_name;
-      ud->outlen_so_far = 0;
-      ud->lines_on_page = 0;
-      ud->braille_page_number = ud->beginning_braille_page_number;
-      ud->print_page_number_first[0] = '_';
-      ud->print_page_number[1] = 0;
-      ud->print_page_number_first[1] = 0;
-      ud->print_page_number_last[0] = 0;
-      ud->page_separator_number_first[0] = 0;
-      ud->page_separator_number_first[0] = 0;
-      ud->fill_pages = 0;
-      ud->mode = mode | ud->config_mode;
-      return 1;
-    }
-  lbu_free ();
   paperWidth = 0;
   paperHeight = 0;
   leftMargin = 0;
   rightMargin = 0;
   topMargin = 0;
   bottomMargin = 0;
-  if (!(ud = malloc (sizeof (UserData))))
+  destroy_semantic_table ();
+  if (fatalErrorCount)
     {
-      lou_logPrint ("liblouisutdml: not enough memory for buffers");
-      lou_logEnd ();
+      configureError (NULL, "%d fatal errors found", fatalErrorCount);
       return 0;
     }
+  if (ud == NULL)
+    {
+      if (!(ud = malloc (sizeof (UserData))))
+        memoryError ();
+        }
   memset (ud, 0, sizeof (UserData));
   ud->outbuf1_len = (sizeof (ud->outbuf1) / CHARSIZE) - 4;
   ud->outbuf2_len = (sizeof (ud->outbuf2) / CHARSIZE) - 4;
@@ -1253,7 +1297,7 @@ read_configuration_file (const char *configFileList, const char
   ud->top = -1;
   ud->style_top = -1;
   ud->emphasis = 15;
-  /* There will be configuration settings for these values*/
+  /* There will be configuration settings for these values */
   ud->cell_width = 5;
   ud->normal_line = 8;
   ud->wide_line = 10;
@@ -1261,7 +1305,7 @@ read_configuration_file (const char *configFileList, const char
   strcpy (ud->lit_hyphen, "-");
   strcpy (ud->comp_hyphen, "_&");
   strcpy (ud->letsign, "\\_");
-  /*End of values*/
+  /*End of values */
   for (k = document; k < notranslate; k++)
     {
       StyleType *style = new_style ((xmlChar *) semNames[k]);
@@ -1270,24 +1314,31 @@ read_configuration_file (const char *configFileList, const char
   ud->input_encoding = utf8;
   ud->output_encoding = ascii8;
   *ud->print_page_number = '_';
+  *ud->print_page_number_first = '_';
   ud->string_escape = ',';
 #ifdef _WIN32
   ud->file_separator = '\\';
 #else
   ud->file_separator = '/';
 #endif
+  pathEnd[0] = ud->file_separator;
+  pathEnd[1] = 0;
 
 /*Process file list*/
   if (configFileList == NULL)
     {
       set_paths (NULL);
-      if (!(config_compileSettings ("liblouisutdml.ini") ||
-	    config_compileSettings ("canonical.cfg")))
+  if (logFileName  && logFileName[0] != 0)
+    {
+      strcpy ((char *) ud->typeform, lbu_getWriteablePath ());
+      strcat ((char *) ud->typeform, logFileName);
+      lou_logFile ((char *) ud->typeform);
+    }
+      if (!(config_compileSettings ("liblouisutdml.ini")))
 	return 0;
     }
   else
     {
-      strcpy (lastConfigFileList, configFileList);
       listLength = strlen (configFileList);
       for (k = 0; k < listLength; k++)
 	if (configFileList[k] == ',')
@@ -1338,15 +1389,14 @@ read_configuration_file (const char *configFileList, const char
     strcat (ud->xml_header, "]>\n");
   ud->mode = mode | ud->config_mode;
   ud->orig_format_for = ud->format_for;
-  if (ud->format_for > utd)
-  {
-  ud->mode &= ~notUC;
-  ud->format_for = utd;
-  }
+  if (ud->format_for > utd || (ud->mode & louisDots))
+    {
+      ud->mode &= ~notUC;
+      ud->format_for = utd;
+    }
   if (ud->format_for == utd)
     {
       ud->braille_pages = 1;
-      ud->paragraphs = 1;
       ud->paper_width = (int) (paperWidth * ud->dpi);
       ud->paper_height = (int) (paperHeight * ud->dpi);
       ud->left_margin = (int) (leftMargin * ud->dpi);
