@@ -128,10 +128,12 @@ semStringHash (const unsigned char *s)
 }
 
 static HashTable *
-hashNew (void)
+hashNew ()
 {
   HashTable *table;
   table = malloc (sizeof (HashTable));
+  if (!table)
+    memoryError ();
   memset (table, 0, sizeof (HashTable));
   table->curBucket = -1;
   return table;
@@ -160,11 +162,11 @@ hashFree (HashTable * table)
 
 static void hashInsert (HashTable * table, const unsigned char *key, int
 			type, int
-			semNum, InsertsType * inserts, StyleType * 
+			semNum, InsertsType * inserts, StyleType *
 			style, unsigned char *macro);
 
 void
-clean_semantic_table (void)
+clean_semantic_table ()
 {
   int i;
   HashEntry *e, *next;
@@ -178,7 +180,7 @@ clean_semantic_table (void)
 	next = e->next;
 	if (!(e->type == 0 || e->type == xpathEntry))
 	  hashInsert (newSemanticTable, e->key, e->type, e->semNum, NULL,
-		      e->style, NULL);
+		      e->style, e->macro);
 	free (e->key);
 	if (e->inserts != NULL)
 	  free (e->inserts);
@@ -228,7 +230,7 @@ static HashEntry *latestEntry;
 /* assumes that key is not already present! */
 static void
 hashInsert (HashTable * table, const unsigned char *key, int type, int
-	    semNum, InsertsType * inserts, StyleType * style, unsigned 
+	    semNum, InsertsType * inserts, StyleType * style, unsigned
 	    char *macro)
 {
   int i;
@@ -236,8 +238,12 @@ hashInsert (HashTable * table, const unsigned char *key, int type, int
     return;
   i = semStringHash (key) % HASHSIZE;
   latestEntry = malloc (sizeof (HashEntry));
+  if (!latestEntry)
+    memoryError ();
   latestEntry->next = table->entries[i];
   latestEntry->key = malloc (strlen ((char *) key) + 2);
+  if (!latestEntry->key)
+    memoryError ();
   strcpy ((char *) latestEntry->key, (char *) key);
   latestEntry->type = type;
   latestEntry->semNum = semNum;
@@ -290,10 +296,12 @@ find_semantic_number (const char *name)
   char realName[MAXNAMELEN];
   if (semanticTable == NULL)
     semanticTable = hashNew ();
+  /* See if semantic names and numbers are already in table. */
   strcpy (realName, "no");
   strcat (realName, SEMANTICSUF);
   if (hashLookup (semanticTable, (xmlChar *) realName) == notFound)
     {
+      /* Enter names and numbers in table before checking for this one. */
       for (k = 0; k < end_all; k++)
 	{
 	  strcpy (realName, semNames[k]);
@@ -311,6 +319,7 @@ find_semantic_number (const char *name)
 	  k++;
 	}
     }
+  /* Now look up the name we called this function with. */
   for (k = 0; name[k]; k++)
     realName[k] = name[k] | 32;
   realName[k] = 0;
@@ -319,7 +328,7 @@ find_semantic_number (const char *name)
 }
 
 void
-destroy_semantic_table (void)
+destroy_semantic_table ()
 {
   hashFree (semanticTable);
   semanticTable = NULL;
@@ -331,7 +340,7 @@ destroy_semantic_table (void)
 }
 
 static widechar
-hexValue (FileInfo * nested, const xmlChar *digits, int length)
+hexValue (FileInfo * nested, const xmlChar * digits, int length)
 {
   int k;
   unsigned int binaryValue = 0;
@@ -476,10 +485,14 @@ encodeInsertions (FileInfo * nested, xmlChar * insertions, int length)
     }
   insertsSize += inserts.numChars * CHARSIZE;
   insertsPtr = malloc (insertsSize);
+  if (!insertsPtr)
+    memoryError ();
   memcpy (insertsPtr, &inserts, insertsSize);
   return insertsPtr;
 }
 
+static int insertFromMacro (int which);
+
 int
 insert_code (xmlNode * node, int which)
 {
@@ -495,6 +508,11 @@ insert_code (xmlNode * node, int which)
   nodeEntry = (HashEntry *) node->_private;
   if (nodeEntry == NULL)
     return 0;
+  if (nodeEntry->macro)
+    {
+      insertFromMacro (which);
+      return 1;
+    }
   if (nodeEntry->inserts == NULL)
     return 1;
   inserts = nodeEntry->inserts;
@@ -544,6 +562,8 @@ countAttrValues (xmlChar * key)
   if (attrValueCounts == NULL)
     {
       attrValueCounts = malloc (NUMCOUNTS * sizeof (int));
+      if (!attrValueCounts)
+	memoryError ();
       attrValueCountsTable = hashNew ();
       curCount = 0;
     }
@@ -562,8 +582,7 @@ countAttrValues (xmlChar * key)
 	return 1;
       if (curCount >= NUMCOUNTS)
 	return 0;
-      hashInsert (attrValueCountsTable, key, 0, curCount, NULL, NULL, 
-      NULL);
+      hashInsert (attrValueCountsTable, key, 0, curCount, NULL, NULL, NULL);
       curCount++;
       return 1;
     case 3:
@@ -574,8 +593,8 @@ countAttrValues (xmlChar * key)
       if (thisCount == notFound)
 	{
 	  attrValueCounts[curCount]++;
-	  hashInsert (attrValueCountsTable, key, 0, curCount, NULL, 
-	  NULL, NULL);
+	  hashInsert (attrValueCountsTable, key, 0, curCount, NULL,
+		      NULL, NULL);
 	  curCount++;
 	}
       key[lastComma] = ',';
@@ -592,7 +611,7 @@ countAttrValues (xmlChar * key)
 }
 
 static void
-destroyattrValueCountsTable (void)
+destroyattrValueCountsTable ()
 {
   if (attrValueCounts == NULL)
     return;
@@ -638,6 +657,7 @@ compileLine (FileInfo * nested)
   char *insertions = NULL;
   int insertionsLength = 0;
   InsertsType *inserts = NULL;
+  unsigned char *macro = NULL;
   StyleType *style = NULL;
   int actionNum = 0;
   if (semanticTable == NULL)
@@ -647,12 +667,12 @@ compileLine (FileInfo * nested)
   if (ch == 0 || ch == '#' || ch == '<')
     return 1;
   action = curchar - 1;
-  while ((ch = *curchar++) > 32);
+  while ((ch = *curchar++) > 32 || ch == '=');
   actionLength = curchar - action - 1;
   action[actionLength] = 0;
   if (!(actionLength == 2 && action[0] == 'n' && action[1] == 'o'))
     nested->unedited = 0;
-  while ((ch = *curchar++) <= 32 && ch != 0);
+  while (((ch = *curchar++) <= 32 || ch == '=') && ch != 0);
   if (ch == 0)
     {
       semanticError (nested, "Nothing to look for");
@@ -717,10 +737,11 @@ compileLine (FileInfo * nested)
   lookFor[lookForLength] = 0;
   actionNum = find_semantic_number (action);
   style = lookup_style (action);
-  if (actionNum == notFound && style == NULL)
+  macro = lookup_macro (action);
+  if (actionNum == notFound && style == NULL && macro == NULL)
     {
       semanticError (nested,
-		     "Action or style '%s' in column 1 not recognized", 
+		     "Action or style or macro '%s' in column 1 not recognized",
 		     action);
       return 0;
     }
@@ -805,7 +826,7 @@ compileLine (FileInfo * nested)
   else
     {
       hashInsert (semanticTable, (xmlChar *) lookFor, type, actionNum,
-		  inserts, style, NULL);
+		  inserts, style, macro);
       nested->numEntries++;
     }
   return 1;
@@ -860,8 +881,7 @@ sem_compileFile (const char *fileName)
     }
   if (!haveAppended && !find_file (fileName, completePath))
     {
-      semanticError (NULL, "Can't find semantic-action file '%s'", 
-      fileName);
+      semanticError (NULL, "Can't find semantic-action file '%s'", fileName);
       haveSemanticFile = 0;
       strcpy (firstFileName, fileName);
       return 0;
@@ -879,13 +899,12 @@ sem_compileFile (const char *fileName)
       fclose (nested.in);
       if (nested.unedited)
 	semanticError (NULL,
-       "File '%s' needs editing to produce good results.",
+		       "File '%s' needs editing to produce good results.",
 		       nested.fileName);
     }
   else
     {
-      semanticError (NULL, "Can't open semantic-action file '%s'", 
-      fileName);
+      semanticError (NULL, "Can't open semantic-action file '%s'", fileName);
       return 0;
     }
   numEntries += nested.numEntries;
@@ -1211,6 +1230,35 @@ get_sem_attr (xmlNode * node)
     return no;
 }
 
+unsigned char *
+get_sem_name (xmlNode * node)
+{
+  int k;
+  static char key[50];
+  HashEntry *nodeEntry = (HashEntry *) node->_private;
+  if (nodeEntry == NULL)
+    return "";
+  if (nodeEntry->macro != NULL)
+    {
+      char *macro = nodeEntry->macro;
+      strcpy (key, "macro,");
+      for (k = 0; macro[k] != ','; k++);
+      k--;
+      strncat (key, macro, k);
+    }
+  else if (nodeEntry->style != NULL)
+  {
+  strcpy (key, "style,");
+    strcat (key, nodeEntry->style->name);
+    }
+  else
+    {
+    strcpy (key, "action,");
+    strcat (key, semNames[nodeEntry->semNum]);
+    }
+  return key;
+}
+
 StyleType *
 is_style (xmlNode * node)
 {
@@ -1221,16 +1269,6 @@ is_style (xmlNode * node)
     return NULL;
 }
 
-char *
-is_macro (xmlNode * node)
-{
-  HashEntry *nodeEntry = (HashEntry *) node->_private;
-  if (nodeEntry != NULL)
-    return nodeEntry->macro;
-  else
-    return NULL;
-}
-
 xmlChar *
 get_attr_value (xmlNode * node)
 {
@@ -1304,7 +1342,7 @@ addNewEntries (const xmlChar * newEntry)
 }
 
 void
-append_new_entries (void)
+append_new_entries ()
 {
   int items;
   char filePrefix[20];
@@ -1360,8 +1398,7 @@ append_new_entries (void)
     lou_logPrint ("%d new entries appended to '%s%s'.", numEntries,
 		  filePrefix, firstFileName);
   else
-    lou_logPrint (
-    "%d entries written to new semantic-action file '%s%s'.",
+    lou_logPrint ("%d entries written to new semantic-action file '%s%s'.",
 		  numEntries, filePrefix, firstFileName);
   moreEntries = 0;
 }
@@ -1398,30 +1435,20 @@ new_style (xmlChar * name)
   strcat (key, STYLESUF);
   if (hashLookup (semanticTable, key) != notFound)
     return latestEntry->style;
-  style = malloc (sizeof (StyleType));
+  style = malloc (sizeof (StyleType) + strlen (name) + 3);
+  if (!style)
+    memoryError ();
   memset (style, 0, sizeof (StyleType));
+  strcpy (style->name, name);
   style->newline_after = 1;
   hashInsert (semanticTable, key, styleEntry, 0, NULL, style, NULL);
+  style->format = inherit;	/* inherit parent format by default */
+  style->right_margin = -100;
+  style->left_margin = -100;
+  style->first_line_indent = -100;
   return style;
 }
 
-#define MACROSUF " orcam"
-xmlChar *
-new_macro (xmlChar * name, xmlChar *body)
-{
-  char *storedBody = alloc_string (body);
-  char key[MAXNAMELEN];
-  if (!semanticTable)
-    semanticTable = hashNew ();
-  strcpy (key, name);
-  strcat (key, MACROSUF);
-  if (hashLookup (semanticTable, key) != notFound)
-    return NULL;
-  hashInsert (semanticTable, key, macroEntry, 0, NULL, NULL, 
-  storedBody);
-  return storedBody;
-}
-
 StyleType *
 lookup_style (xmlChar * name)
 {
@@ -1433,6 +1460,54 @@ lookup_style (xmlChar * name)
   return NULL;
 }
 
+StyleType *
+action_to_style (sem_act action)
+{
+  if (action < 0 || action >= end_all)
+    return NULL;
+  return lookup_style ((char *) semNames[action]);
+}
+
+/* Beginning of macro processing */
+
+/* Hold macro state */
+static char macroName[40];
+static char *macro = NULL;
+static int macroLength = 0;
+static int posInMacro;;
+static xmlNode *macroNode;
+static HashEntry *isMacroEntry;
+static int macroHasStyle;
+static char *paramStart = NULL;
+static int paramLength = 0;
+
+static void
+macroError (char *format, ...)
+{
+  char buffer[MAXNAMELEN];
+  va_list arguments;
+  va_start (arguments, format);
+#ifdef WIN32
+  _vsnprintf (buffer, sizeof (buffer), format, arguments);
+#else
+  vsnprintf (buffer, sizeof (buffer), format, arguments);
+#endif
+  va_end (arguments);
+  lou_logPrint ("Macro %s: %s", macroName, buffer);
+}
+
+char *
+is_macro (xmlNode * node)
+{
+  HashEntry *nodeEntry = (HashEntry *) node->_private;
+  if (nodeEntry != NULL)
+    return nodeEntry->macro;
+  else
+    return NULL;
+}
+
+#define MACROSUF " orcam"
+
 unsigned char *
 lookup_macro (xmlChar * name)
 {
@@ -1444,10 +1519,301 @@ lookup_macro (xmlChar * name)
   return NULL;
 }
 
-StyleType *
-action_to_style (sem_act action)
+xmlChar *
+new_macro (xmlChar * name, xmlChar * body)
 {
-  if (action < 0 || action >= end_all)
+  char *storedMacro;
+  char key[MAXNAMELEN];
+  xmlChar *nameAndBody = (xmlChar *) ud->text_buffer;
+  strcpy (nameAndBody, name);
+  strcat (nameAndBody, ",");
+  strcat (nameAndBody, body);
+  storedMacro = alloc_string (nameAndBody);
+  if (!semanticTable)
+    semanticTable = hashNew ();
+  strcpy (key, name);
+  strcat (key, MACROSUF);
+  if (hashLookup (semanticTable, key) != notFound)
     return NULL;
-  return lookup_style ((char *) semNames[action]);
+  hashInsert (semanticTable, key, macroEntry, 0, NULL, NULL, storedMacro);
+  return storedMacro;
 }
+
+static int
+insertFromMacro (int which)
+{
+  return 1;
+}
+
+static int
+doSemanticActions ()
+{
+  int retVal = 1;
+  int semNum = atoi (&macro[posInMacro]);
+  for (; isdigit (macro[posInMacro]) && posInMacro < macroLength;
+       posInMacro++);
+  if (macro[posInMacro] == '(')
+    {
+      paramStart = &macro[++posInMacro];
+      paramLength = find_group_length ("()", paramStart - 1);
+      posInMacro += paramLength;
+      paramLength -= 2;
+    }
+  switch (semNum)
+    {
+    case no:
+      if (ud->text_length > 0 && ud->text_length < MAX_LENGTH
+	  && ud->text_buffer[ud->text_length - 1] > 32)
+	ud->text_buffer[ud->text_length++] = 32;
+      break;
+    case skip:
+      retVal = -1;
+      break;
+    case markhead:
+      ud->head_node = macroNode;
+      break;
+    case pagebreak:
+      do_pagebreak (macroNode);
+      break;
+    case attrtotext:
+      do_attrtotext (macroNode);
+      break;
+    case configtweak:
+      {
+	int k;
+	int kk = 0;
+	xmlChar configString[2 * MAXNAMELEN];
+	configString[kk++] = ud->string_escape;
+	for (k = 0; k < paramLength; k++)
+	  {
+	    if (paramStart[k] == '=')
+	      configString[kk++] = ' ';
+	    else if (paramStart[k] == ';')
+	      configString[kk++] = '\n';
+	    else
+	      configString[kk++] = (xmlChar) paramStart[k];
+	  }
+	configString[kk++] = '\n';
+	configString[kk] = 0;
+	if (!config_compileSettings ((char *) configString))
+	  return 0;
+      }
+      ud->main_braille_table = ud->contracted_table_name;
+      if (!lou_getTable (ud->main_braille_table))
+	{
+	  lou_logPrint ("Cannot open main table %s", ud->main_braille_table);
+	  return 0;
+	}
+      break;
+    case htmllink:
+      if (ud->format_for != browser)
+	break;
+      insert_linkOrTarget (macroNode, 0);
+      break;
+    case htmltarget:
+      if (ud->format_for != browser)
+	break;
+      insert_linkOrTarget (macroNode, 1);
+      break;
+    case boxline:
+      do_boxline (macroNode);
+      break;
+    case blankline:
+      do_blankline ();
+      break;
+    case linespacing:
+      do_linespacing (macroNode);
+      break;
+    case softreturn:
+      do_softreturn ();
+      break;
+    case righthandpage:
+      do_righthandpage ();
+      break;
+    case code:
+      transcribe_computerCode (macroNode, 0);
+      break;
+    case math:
+      transcribe_math (macroNode, 0);
+      break;
+    case graphic:
+      transcribe_graphic (macroNode, 0);
+      break;
+    case chemistry:
+      transcribe_chemistry (macroNode, 0);
+      break;
+    case music:
+      transcribe_music (macroNode, 0);
+      break;
+    case changetable:
+      change_table (macroNode);
+      break;
+    case pagenum:
+      do_pagenum ();
+      break;
+    default:
+      macroError ("semantic action %s is not supported in a macro",
+		  semNames[semNum]);
+      retVal = 0;
+    }
+  return retVal;
+}
+
+static int
+compileMacro ()
+{
+  xmlChar compiledMacro[4 * MAXNAMELEN];
+  int unPos = 0;
+  int pos = 0;
+  memset (compiledMacro, sizeof (compiledMacro), 0);
+  while (unPos < macroLength)
+    {
+      if (isalpha (macro[unPos]))
+	{
+	  char name[40];
+	  int k;
+	  StyleType *style;
+	  for (k = 0; isalnum (macro[unPos]) && unPos < macroLength; unPos++)
+	    name[k++] = macro[unPos];
+	  name[k] = 0;
+	  if ((style = lookup_style (name)) != NULL)
+	    {
+	      if (macroHasStyle)
+		{
+		  macroError ("only one style can be specified");
+		  compiledMacro[0] = '!';
+		  break;
+		}
+	      else
+		{
+		  macroHasStyle = 1;
+		  compiledMacro[pos++] = '~';
+		  isMacroEntry->style = style;
+		}
+	    }
+	  else
+	    {
+	      if ((k = find_semantic_number (name)) != notFound)
+		{
+		  k = sprintf (name, "%d", k);
+		  strcpy (&compiledMacro[pos], name);
+		  pos += k;
+		  if (macro[unPos] == '(')
+		    {
+		      k = find_group_length ("()", &macro[unPos]);
+		      strncpy (&compiledMacro[pos], &macro[unPos], k);
+		      pos += k;
+		      unPos += k;
+		    }
+		}
+	      else if (strcmp (name, "styleend") == 0)
+		compiledMacro[pos++] = '@';
+	      else if (strcmp (name, "pause") == 0)
+		compiledMacro[pos++] = '#';
+	      else
+		{
+		  macroError
+		    ("'%s' is neither a style or semantic action or keyword",
+		     name);
+		  compiledMacro[0] = '!';
+		  break;
+		}
+	    }
+	}
+      if (macro[unPos] == ',')
+	compiledMacro[pos++] = macro[unPos++];
+    }
+  compiledMacro[pos] = 0;
+  strcpy (macro, compiledMacro);
+  macroLength = pos;
+  return 1;
+}
+
+static int
+executeMacro ()
+{
+  while (posInMacro < macroLength)
+    {
+      if (isdigit (macro[posInMacro]))
+	doSemanticActions ();
+      else
+	switch (macro[posInMacro])
+	  {
+	  case ',':
+	    posInMacro++;
+	    break;
+	  case '~':
+	    start_style (isMacroEntry->style, macroNode);
+	    macroHasStyle = 1;
+	    posInMacro++;
+	    break;
+	  case '@':
+	    end_style ();
+	    macroHasStyle = 0;
+	    posInMacro++;
+	    break;
+	  case '#':
+	    /* Pause for calling function to do something. */
+	    posInMacro++;
+	    return 1;
+	  default:
+	    posInMacro++;
+	    break;
+	  }
+    }
+  if (posInMacro >= macroLength)
+    {
+      if (macroHasStyle)
+	end_style ();
+      macro = NULL;
+    }
+}
+
+int
+start_macro (xmlNode * node)
+{
+  /* Set macro state */
+  int k;
+  macroNode = node;
+  isMacroEntry = (HashEntry *) node->_private;
+  if (isMacroEntry == NULL || isMacroEntry->macro == NULL)
+    return 0;
+  macro = isMacroEntry->macro;
+  for (k = 0; macro[k] != ','; k++)
+    macroName[k] = macro[k];
+  macroName[k] = 0;
+  if (macroLength != 0)
+    {
+      macroError ("macros cannot be nested");
+      return 0;
+    }
+  macro = &macro[k + 1];
+  macroLength = strlen (macro);
+  posInMacro = 0;
+  macroHasStyle = 0;
+  paramStart = NULL;
+  paramLength = 0;
+  /*compile macro the first time it is used. */
+  if (isalpha (macro[0]))
+    compileMacro ();
+  if (macro[0] == '!')
+    /* Contains errors */
+    return 0;
+  executeMacro ();
+  return macroHasStyle;
+}
+
+int
+end_macro ()
+{
+  if (macro == NULL || macro[0] == '!')
+    {
+      macroLength = 0;
+      return 0;
+    }
+  executeMacro ();
+  macroLength = 0;
+  return 1;
+}
+
+/* End of macro processing */
