@@ -43,21 +43,26 @@ static int orphan_control = 0;
 static int orphan_control_pos = 0;
 static int orphan_control_status = 0;
 static int saved_text_length;
+static int saved_sync_text_length;
 static int saved_translated_length;
 static int saved_outbuf1_len_so_far;
 static int saved_outbuf2_len_so_far;
 static int saved_running_head_length;
 static int saved_footer_length;
 static int saved_braille_page_number;
-static int saved_brl_page_num_format;
-static int saved_cur_brl_page_num_format;
+static int saved_style_left_margin;
+static int saved_style_right_margin;
+static int saved_style_first_line_indent;
+static StyleFormat saved_style_format;
+static BrlPageNumFormat saved_brl_page_num_format;
+static BrlPageNumFormat saved_cur_brl_page_num_format;
 static int saved_lines_on_page;
 static int saved_line_spacing;
 static int saved_blank_lines;
 static int saved_fill_pages;
-static int saved_fill_page_skipped;
-static char saved_soft_hyphens[2 * BUFSIZE];
+static int saved_positions_array[2 * BUFSIZE];
 static widechar saved_text_buffer[2 * BUFSIZE];
+static widechar saved_sync_text_buffer[2 * BUFSIZE];
 static widechar saved_translated_buffer[2 * BUFSIZE];
 static widechar saved_outbuf1[2 * BUFSIZE];
 static widechar saved_outbuf2[2 * BUFSIZE];
@@ -75,11 +80,16 @@ static void
 saveState (void)
 {
   saved_text_length = ud->text_length;
+  saved_sync_text_length = ud->sync_text_length;
   saved_translated_length = ud->translated_length;
   saved_outbuf1_len_so_far = ud->outbuf1_len_so_far;
   saved_outbuf2_len_so_far = ud->outbuf2_len_so_far;
   saved_running_head_length = ud->running_head_length;
   saved_footer_length = ud->footer_length;
+  saved_style_left_margin = ud->style_left_margin;
+  saved_style_right_margin = ud->style_right_margin;
+  saved_style_first_line_indent = ud->style_first_line_indent;
+  saved_style_format = ud->style_format;
   saved_braille_page_number = ud->braille_page_number;
   saved_brl_page_num_format = ud->brl_page_num_format;
   saved_cur_brl_page_num_format = ud->cur_brl_page_num_format;
@@ -87,8 +97,11 @@ saveState (void)
   saved_line_spacing = ud->line_spacing;
   saved_blank_lines = ud->blank_lines;
   saved_fill_pages = ud->fill_pages;
-  saved_fill_page_skipped = ud->fill_page_skipped;
+  memcpy (saved_positions_array, ud->positions_array,
+	  saved_translated_length * sizeof (int));
   widecharcpy (saved_text_buffer, ud->text_buffer, saved_text_length);
+  widecharcpy (saved_sync_text_buffer, ud->sync_text_buffer,
+	       saved_sync_text_length);
   widecharcpy (saved_translated_buffer, ud->translated_buffer,
 	       saved_translated_length);
   widecharcpy (saved_outbuf1, ud->outbuf1, saved_outbuf1_len_so_far);
@@ -106,18 +119,24 @@ saveState (void)
   widestrcpy (saved_print_page_number_last, ud->print_page_number_last);
   widestrcpy (saved_print_page_number, ud->print_page_number);
   widestrcpy (saved_braille_page_string, ud->braille_page_string);
-  savePointers ();
+  save_translated_buffer ();
+  contents_save_state ();
 }
 
 static void
 restoreState (void)
 {
   ud->text_length = saved_text_length;
+  ud->sync_text_length = saved_sync_text_length;
   ud->translated_length = saved_translated_length;
   ud->outbuf1_len_so_far = saved_outbuf1_len_so_far;
   ud->outbuf2_len_so_far = saved_outbuf2_len_so_far;
   ud->running_head_length = saved_running_head_length;
   ud->footer_length = saved_footer_length;
+  ud->style_left_margin = saved_style_left_margin;
+  ud->style_right_margin = saved_style_right_margin;
+  ud->style_first_line_indent = saved_style_first_line_indent;
+  ud->style_format = saved_style_format;
   ud->braille_page_number = saved_braille_page_number;
   ud->brl_page_num_format = saved_brl_page_num_format;
   ud->cur_brl_page_num_format = saved_cur_brl_page_num_format;
@@ -125,8 +144,11 @@ restoreState (void)
   ud->line_spacing = saved_line_spacing;
   ud->blank_lines = saved_blank_lines;
   ud->fill_pages = saved_fill_pages;
-  ud->fill_page_skipped = saved_fill_page_skipped;
+  memcpy (ud->positions_array, saved_positions_array,
+	  saved_translated_length * sizeof (int));
   widecharcpy (ud->text_buffer, saved_text_buffer, saved_text_length);
+  widecharcpy (ud->sync_text_buffer, saved_sync_text_buffer,
+	       saved_sync_text_length);
   widecharcpy (ud->translated_buffer, saved_translated_buffer,
 	       saved_translated_length);
   widecharcpy (ud->outbuf1, saved_outbuf1, saved_outbuf1_len_so_far);
@@ -144,7 +166,8 @@ restoreState (void)
   widestrcpy (ud->print_page_number_last, saved_print_page_number_last);
   widestrcpy (ud->print_page_number, saved_print_page_number);
   widestrcpy (ud->braille_page_string, saved_braille_page_string);
-  restorePointers ();
+  restore_translated_buffer ();
+  contents_restore_state ();
 }
 
 int
@@ -159,6 +182,7 @@ transcribe_paragraph (xmlNode * node, int action)
   int keep_with_previous_this = 0;
   int orphan_control_this = 0;
   StyleType *style;
+  int haveMacro = 0;
   xmlNode *child;
   int branchCount = 0;
   int i;
@@ -189,10 +213,10 @@ transcribe_paragraph (xmlNode * node, int action)
       if (!lou_getTable (ud->main_braille_table))
 	{
 	  lou_logPrint ("Cannot open main table %s", ud->main_braille_table);
-	  kill_safely ();
+	  return 0;
 	}
       if (action != 0)
-        pop_sem_stack ();
+	pop_sem_stack ();
       return 1;
     case htmllink:
       if (ud->format_for != browser)
@@ -213,6 +237,16 @@ transcribe_paragraph (xmlNode * node, int action)
       if (action != 0)
 	pop_sem_stack ();
       return 1;
+    case pagebreak:
+      do_pagebreak (node);
+      if (action != 0)
+	pop_sem_stack ();
+      return 1;
+    case attrtotext:
+      do_attrtotext (node);
+      if (action != 0)
+	pop_sem_stack ();
+      return 1;
     case blankline:
       do_blankline ();
       if (action != 0)
@@ -228,6 +262,11 @@ transcribe_paragraph (xmlNode * node, int action)
       if (action != 0)
 	pop_sem_stack ();
       return 1;
+    case newpage:
+      do_newpage ();
+      if (action != 0)
+	pop_sem_stack ();
+      return 1;
     case righthandpage:
       do_righthandpage ();
       if (action != 0)
@@ -267,7 +306,12 @@ transcribe_paragraph (xmlNode * node, int action)
     default:
       break;
     }
-  if ((style = is_style (node)) != NULL)
+  if (is_macro (node))
+    {
+      haveMacro = 1;
+      start_macro (node);
+    }
+  else if ((style = is_style (node)) != NULL)
     start_style (style, node);
   child = node->children;
   while (child)
@@ -337,7 +381,11 @@ transcribe_paragraph (xmlNode * node, int action)
 	    transcribe_paragraph (child, 1);
 	  break;
 	case XML_TEXT_NODE:
-	  insert_text (child);
+	  /*Is there already a <brl> node? */
+	  if (!(ud->format_for == utd && child->next != NULL
+		&& strcmp ((char *) child->next->name, "brl") == 0))
+	    insert_text (child);
+	  /*Is there now a <brl>node? */
 	  if (ud->format_for == utd && child->next != NULL
 	      && strcmp ((char *) child->next->name, "brl") == 0)
 	    child = child->next;	/*skip <brl> node */
@@ -470,46 +518,22 @@ transcribe_paragraph (xmlNode * node, int action)
     }
   insert_code (node, branchCount);
   insert_code (node, -1);
-  if (style)
+  if (haveMacro)
+    end_macro ();
+  else if (style)
     end_style ();
   else
     switch (ud->stack[ud->top])
       {
       case runninghead:
-	if (ud->format_for == utd)
-	  {
-	    memcpy (ud->running_head, ud->text_buffer, ud->text_length *
-		    CHARSIZE);
-	    ud->running_head_length = ud->text_length;
-	  }
-	else
-	  {
-	    insert_translation (ud->main_braille_table);
-	    if (ud->translated_length > (ud->cells_per_line - 9))
-	      ud->running_head_length = ud->cells_per_line - 9;
-	    else
-	      ud->running_head_length = ud->translated_length;
-	    memcpy (&ud->running_head[0], &ud->translated_buffer[0],
-		    ud->running_head_length * CHARSIZE);
-	  }
+	insert_translation (ud->main_braille_table);
+	set_runninghead_string (ud->translated_buffer, ud->translated_length);
+	ud->translated_length = 0;
 	break;
       case footer:
-	if (ud->format_for == utd)
-	  {
-	    memcpy (ud->footer, ud->text_buffer, ud->text_length * 
-	    CHARSIZE);
-	    ud->footer_length = ud->text_length;
-	  }
-	else
-	  {
-	    insert_translation (ud->main_braille_table);
-	    if (ud->translated_length > (ud->cells_per_line - 9))
-	      ud->footer_length = ud->cells_per_line - 9;
-	    else
-	      ud->footer_length = ud->translated_length;
-	    memcpy (&ud->footer[0], &ud->translated_buffer[0],
-		    ud->footer_length * CHARSIZE);
-	  }
+	insert_translation (ud->main_braille_table);
+	set_footer_string (ud->translated_buffer, ud->translated_length);
+	ud->translated_length = 0;
 	break;
       default:
 	break;
